/*
   File:   Dds.cpp

     File:   Dds.h
   Author: Anthony (f4goh@orange.fr)

   Created on 20 juin 2024
   //sortie PWM sur OCR2A D11 CH2 et OCR2B D3 CH1
   //Fech sur D2

*/

#include "Dds.h"

/**
  @param
   Le Constructeur Dds
   _slpFreq : fréquence d'échantillonage
*/

Dds::Dds()
  : dephase1(0),
    dephase2(0),
    incrementPhase1(0),
    incrementPhase2(0),
    compteur(0),
    ddsState(0) {
  anchor = this;
}

Dds::Dds(const Dds& orig) {
}

Dds::~Dds() {
}

/**
   @brief Dds::begin()
   @details Permet d'initialiser le dds
*/

void Dds::begin() {
  uint16_t n;
  for (n = 0; n < 256; n++) {
    sinusTable[n] = 127 + 127 * sin(PI * n / 128.0);
  }

  DDRD = 254;
  DDRB = 255;

  TCCR2A = 0xA3; //timer 0 fast PWm at PD5/6
  TCCR2B = 0x01; //16Mhz
  TIMSK2 = 0x01; //interruption Timer 2 Overflow
  OCR2A = 128;
  OCR2B = 128;


  accumulateur1 = 0;
  accumulateur2 = 0;
  sei();
}

/**
   @brief    Dds::off()
   @details  arrête la génération du signal
*/

void Dds::off() {
  cli();
}


void Dds::marshall() {
  anchor->interruption();
}

/**
   @brief Dds::interuption()
   @details interruption générant la sinusoide vers le dac
   voir fiche principe du dds
*/

void Dds::interruption() {

  uint8_t phase1, phase2;
  PORTD |= SYNC;
  if (ddsState) {
    accumulateur1 += incrementPhase1;           // accumulateur de phase  (sur 16 bits)
    phase1 = ((accumulateur1 >> 8)  + dephase1);  //ajoute la phase
    OCR2A = sinusTable[phase1] / ATTENUATION ;                //lecture de la valeur du sinus dans la table
    accumulateur2 += incrementPhase2;           // accumulateur de phase  (sur 16 bits)
    phase2 = ((accumulateur2 >> 8)  + dephase2);  //ajoute la phase
    OCR2B = sinusTable[phase2] / ATTENUATION;                //lecture de la valeur du sinus dans la table
  }
  compteur++;
  PORTD &= ~SYNC;
}


/**
   @brief Dds::computeIncrementPhase(float freq)
   @details calcul le mot utilisé par le DDS pour générer une sinusoide à fréquence donnée
   @param   float la frequence en hz
*/

uint16_t Dds::computeIncrementPhase(float freq) {
  return pow(2, 16) * freq / SAMPLING_FREQUENCY;
}

/**
   @brief Dds::setFrequency(float freq)
   @details met a jour la fréquence de sortie du dac
   @param   float la frequence en hz
*/

void Dds::setFrequency(float freq1, float freq2) {
  incrementPhase1 = computeIncrementPhase(freq1);
  incrementPhase2 = computeIncrementPhase(freq2);
}


/**
   @brief Dds::start()
   @details démarre le dds
*/

void Dds::start() { //différencier les 2 dds
  ddsState = true;
}

/**
   @brief Dds::stop()
   @details stop le dds
*/

void Dds::stop() {
  ddsState = false;
}


/**
   @brief setPhase(int ph
   @details met a jour la phase du dds
   @param   int la phase entre 0 et 359°
*/

void Dds::setPhase(uint16_t ph1, uint16_t ph2) {
  dephase1 = round(ph1 * 256 / 360);
  dephase2 = round(ph2 * 256 / 360);
}

void Dds::setIQ() {
  dephase1 = 0;
  dephase2 = round(90 * 256 / 360);
}

void Dds::delay(int ms) {
  uint16_t ticks;
  if (ms > 1000) ms = 1000;
  ticks = ms * SAMPLING_FREQUENCY / 1000;
  compteur = 0;
  while (compteur < ticks) {
  }
}

ISR (TIMER2_OVF_vect) {
  Dds::marshall();
}


Dds* Dds::anchor = NULL;
